Package: readstata13
Type: Package
Title: Import 'Stata' Data Files
Version: 0.11.0
Authors@R: c(
    person("Jan Marvin", "Garbuszus",
    email = "jan.garbuszus@ruhr-uni-bochum.de", role = c("aut")),
    person("Sebastian", "Jeworutzki",
    email="Sebastian.Jeworutzki@ruhr-uni-bochum.de", role = c("aut", "cre"), comment = c(ORCID = "0000-0002-2671-5253")),
    person("R Core Team", role="cph"),
    person("Magnus Thor", "Torfason", role="ctb"),
    person("Luke M.", "Olson", role="ctb"),
    person("Giovanni", "Righi", role="ctb"),
    person("Kevin", "Jin", role="ctb")
    )
Description: Function to read and write the 'Stata' file format.
URL: https://github.com/sjewo/readstata13
BugReports: https://github.com/sjewo/readstata13/issues
License: GPL-2 | file LICENSE
Imports: Rcpp (>= 0.11.5)
LinkingTo: Rcpp
ByteCompile: yes
Suggests: testthat, knitr, rmarkdown, curl, png, expss, labelled
VignetteBuilder: knitr
Encoding: UTF-8
RoxygenNote: 7.3.2
NeedsCompilation: yes
Packaged: 2025-04-25 10:18:03 UTC; sj
Author: Jan Marvin Garbuszus [aut],
  Sebastian Jeworutzki [aut, cre]
    (<https://orcid.org/0000-0002-2671-5253>),
  R Core Team [cph],
  Magnus Thor Torfason [ctb],
  Luke M. Olson [ctb],
  Giovanni Righi [ctb],
  Kevin Jin [ctb]
Maintainer: Sebastian Jeworutzki <Sebastian.Jeworutzki@ruhr-uni-bochum.de>
Repository: CRAN
Date/Publication: 2025-04-25 11:20:02 UTC
Built: R 4.6.0; aarch64-apple-darwin23; 2026-04-25 16:18:48 UTC; unix
Archs: readstata13.so.dSYM
