# Generated by roxygen2: do not edit by hand

export("varlabel<-")
export(as.caldays)
export(get.frames)
export(get.label)
export(get.label.name)
export(get.label.tables)
export(get.lang)
export(get.origin.codes)
export(read.dta13)
export(read.dtas)
export(save.dta13)
export(set.label)
export(set.lang)
export(stbcal)
export(varlabel)
import(Rcpp)
importFrom(stats,complete.cases)
importFrom(stats,na.omit)
importFrom(stats,setNames)
importFrom(utils,download.file)
importFrom(utils,setTxtProgressBar)
importFrom(utils,txtProgressBar)
importFrom(utils,unzip)
useDynLib(readstata13, .registration = TRUE)
