#' Import Stata Data Files
#'
#' Function to read the Stata file format into a data.frame.
#'
#'
#' @author Marvin Garbuszus \email{jan.garbuszus@@ruhr-uni-bochum.de}
#' @author Sebastian Jeworutzki \email{sebastian.jeworutzki@@ruhr-uni-bochum.de}
#'
#' @name readstata13
#' @docType package
#' @useDynLib readstata13, .registration = TRUE
#' @import Rcpp
#' @note If you catch a bug, please do not sue us, we do not have any money.
#' @seealso \code{\link[foreign]{read.dta}} and \code{memisc} for dta files from
#'  Stata Versions < 13
NULL
