# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

stata_read <- function(filePath, missing, selectrows, selectcols, strlexport, strlpath) {
    .Call(`_readstata13_stata_read`, filePath, missing, selectrows, selectcols, strlexport, strlpath)
}

stata_save <- function(filePath, dat) {
    .Call(`_readstata13_stata_save`, filePath, dat)
}

stata_pre13_save <- function(filePath, dat) {
    .Call(`_readstata13_stata_pre13_save`, filePath, dat)
}

